import de421
from jplephem import Ephemeris
import astropy.io.fits as pyfits
from numpy import *



c = 299792.458 # km/s  (kilometers per second)


def barycor(date,ra,dec, orbit=False, return_correction=True):
    """Apply barycentric correction to vector 
       date (in MJD) assuming coordinates ra,dec given in degrees
       optionally use orbit of a satellinte in heasarc format
       This includes geometric and shapiro corrections, but einstein
       correction is not implemented for now. Accurate to about 3e-4s.
       If return_correction=True (default), correction in seconds is returned.
    v0.1: 20.02.2020 by V. Doroshenko
       """
    ra = radians(double(ra))
    dec = radians(double(dec))
    jd =  date + 2400000.5
    # emrat = 81.3
    emrat1 = 0.012150586

    # read ephemeris and read position and velocities of earth/moon barycenter and moon
    eph = Ephemeris(de421)
    msol = eph.GMS
    (x_em,y_em,z_em),(vx_em,vy_em,vz_em) = eph.position_and_velocity('earthmoon',jd)
    (x_m,y_m,z_m),(vx_m,vy_m,vz_m) = eph.position_and_velocity('moon',jd)
    (x_sun,y_sun,z_sun),(vx_sun,vy_sun,vz_sun) = eph.position_and_velocity('sun',jd)

    #calculate position and velocity of the earth
    x_earth = x_em-emrat1*x_m
    y_earth = y_em-emrat1*y_m
    z_earth = z_em-emrat1*z_m

    vx_earth = vx_em-emrat1*vx_m
    vy_earth = vy_em-emrat1*vy_m
    vz_earth = vz_em-emrat1*vz_m
    
    if orbit:
        orbit = pyfits.open(orbit)
        mjdref = orbit[1].header['mjdref']#+orbit[1].header['mjdrefi']
        try:            
            minmet = (min(date)-1.-mjdref)*86400
            maxmet =(max(date)+1.-mjdref)*86400
        except:
            minmet = (date-1-mjdref)*86400
            maxmet = (date+1-mjdref)*86400
        try:
            t = orbit[1].data.field('time')
        except:
            t = orbit[1].data.field('sclk_utc')
        mi, ma = t.searchsorted([minmet,maxmet])
        t = t[mi:ma]/86400.+mjdref
        
        # interpolate orbit to observed time and convert to km and km/s
        try:            
            x_s = interp(date, t, orbit[1].data.field('x')[mi:ma]/1000.)
            y_s = interp(date, t, orbit[1].data.field('y')[mi:ma]/1000.)
            z_s = interp(date, t, orbit[1].data.field('z')[mi:ma]/1000.)
        
            vx_s = interp(date, t, orbit[1].data.field('vx')[mi:ma]/1000.)
            vy_s = interp(date, t, orbit[1].data.field('vy')[mi:ma]/1000.)
            vz_s = interp(date, t, orbit[1].data.field('vz')[mi:ma]/1000.)
        except:
            x_s = interp(date, t, orbit[1].data.field('pos_x')[mi:ma]/1000.)
            y_s = interp(date, t, orbit[1].data.field('pos_y')[mi:ma]/1000.)
            z_s = interp(date, t, orbit[1].data.field('pos_z')[mi:ma]/1000.)
        
            vx_s = interp(date, t, orbit[1].data.field('vel_x')[mi:ma]/1000.)
            vy_s = interp(date, t, orbit[1].data.field('vel_y')[mi:ma]/1000.)
            vz_s = interp(date, t, orbit[1].data.field('vel_z')[mi:ma]/1000.)
            
            
        
        x_obs, y_obs, z_obs = x_earth + x_s, y_earth + y_s, z_earth + z_s
        vx_obs, vy_obs, vz_obs = vx_earth + vx_s, vy_earth + vy_s, vz_earth + vz_s
        ocor = (vx_earth*vx_s+vy_earth*vy_s+vz_earth*vz_s)/c**2
    else:
        x_obs, y_obs, z_obs = x_earth, y_earth , z_earth
        vx_obs, vy_obs, vz_obs = vx_earth , vy_earth , vz_earth
        ocor = 0.
        
    # #components of the object unit vector:
    x_obj = cos(dec)*cos(ra)
    y_obj = cos(dec)*sin(ra)
    z_obj = sin(dec)

    #geometric correction
    geo_corr = (x_obs*x_obj + y_obs*y_obj + z_obs*z_obj)/c

    #einstein correction
    # einstein_corr = tdb2tdt(jd, tbase=2400000.5)

    #shapiro correction
    sun_dist = sqrt((x_sun-x_obs)**2+(y_sun-y_obs)**2+(z_sun-z_obs)**2)
    costh = ((x_obs-x_sun)*x_obj+(y_obs-y_sun)*y_obj + (z_obs-z_sun)*z_obj)/sun_dist
    shapiro_corr =  2*msol * log(1.+costh)
    corr = geo_corr + ocor + shapiro_corr # + einstein_corr
    if return_correction:
        return corr
    else:
        return date + corr/86400.
    
        
